local debug = false
function debugPrint(msg)
    if debug then
        print(msg)
    end
end

local ESX = nil
local QBCore = nil

if Config.Framework == "ESX" then
    if GetResourceState("es_extended") ~= "started" then
        print("[^1ERROR^7] es_extended is not started, please start it before starting this resource.")
    end
    ESX = exports["es_extended"]:getSharedObject()
elseif Config.Framework == "QBCORE" then
    if GetResourceState("qb-core") ~= "started" then
        print("[^1ERROR^7] qb-core is not started, please start it before starting this resource.")
    end
    if GetResourceState("qb-banking") ~= "started" then
        print("[^1ERROR^7] qb-banking is not started, please start it before starting this resource.")
    end
    QBCore = exports["qb-core"]:GetCoreObject()
end

local function reward(player)
    debugPrint(player, Config.BasicReward, Config.MoneyEarned, Config.Framework)
    if Config.BasicReward then
        if Config.Framework == "ESX" then
            local xPlayer = ESX.GetPlayerFromId(player)
            xPlayer.addMoney(Config.MoneyEarned)
            debugPrint("ESX", Config.MoneyEarned)
        elseif Config.Framework == "QBCORE" then
            local xPlayer = QBCore.Functions.GetPlayer(player)
            xPlayer.Functions.AddMoney("cash", Config.MoneyEarned, "vote-reward")
            debugPrint("QBCORE", Config.MoneyEarned)
        end
        TriggerClientEvent("sp-vote:helpNotification", player, ("Merci de votre Vote ! Récompense : ~g~$%s~s~"):format(Config.MoneyEarned))
    end
end

local players = GetPlayers()
local processedVotes = {}

local function IsVoteProcessed(vote)
    local diff = os.time() - vote.voted_at
    for _, processedVote in ipairs(processedVotes) do
        if (processedVote.voted_at == vote.voted_at and processedVote.username == vote.username) or diff > 5400 then
            return true
        end
    end
    return false
end

local function verifRpNames(names)
    debugPrint("verifRpNames")
    local resultReturn = nil
    if Config.Framework == "ESX" then
        debugPrint("ESX")
        MySQL.Async.fetchAll("SELECT * FROM users WHERE CONCAT(firstname, ' ', lastname) = @names", {
            ["@names"] = names
        }, function(result)
            if result[1] ~= nil then
                resultReturn = true
            else
                resultReturn = false
            end
        end)
    elseif Config.Framework == "QBCORE" then
        debugPrint("QBCORE")
        MySQL.Async.fetchAll("SELECT * FROM players", {}, function(result)
            for k, v in pairs(result) do
                debugPrint("names", names)
                debugPrint("v.name", v.name)
                local charinfo = json.decode(v.charinfo)
                if (string.format("%s %s", charinfo.firstname:lower(), charinfo.lastname:lower()) == names) or v.name:lower() == names then
                    resultReturn = true
                else
                    resultReturn = false
                end
            end
        end)
    end
    local count = 0
    while not resultReturn do
        count = count + 1
        Citizen.Wait(1000)
        if count > 5 then
            break
        end
    end
    return resultReturn
end
verifRpNames("foltone")
local function ProcessVote(vote)
    debugPrint("ProcessVote")
    local playerFound = false
    players = GetPlayers()
    if #players > 0 then
        for k, v in pairs(players) do
            local playerName = GetPlayerName(tonumber(v))
            if (vote.username:lower() == playerName:lower()) then
                debugPrint("Players online")
                TriggerEvent("sp-vote:newVoteOnline", tonumber(v))
                debugPrint(("New Vote Online - Voted at: %s - Username: %s"):format(vote.voted_at, vote.username))
                playerFound = true
                break
            end
        end
        if verifRpNames(vote.username:lower()) then
            playerFound = true
        end
    end
    if not playerFound then
        debugPrint("Players ofline")
        TriggerEvent("sp-vote:newVoteOffline", vote)
    end
end

local function FetchVotes()
    debugPrint("FetchVotes")
    PerformHttpRequest(("https://serveur-prive.net/api/v1/servers/%s/votes"):format(Config.Token), function(errorCode, resultData, resultHeaders)
        debugPrint("PerformHttpRequest", errorCode, resultData, resultHeaders)
        if errorCode ~= nil and errorCode == 200 then
            if resultData ~= nil then
                local resultData = json.decode(resultData)
                if type(resultData) == "table" and type(resultData.data) == "table" then
                    for _, vote in ipairs(resultData.data) do
                        if type(vote) == "table" and type(vote.voted_at) == "number" and type(vote.username) == "string" then
                            debugPrint(("Vote detect - Voted at: %s - Username: %s"):format(vote.voted_at, vote.username))
                            if not IsVoteProcessed(vote) then
                                ProcessVote(vote)
                                table.insert(processedVotes, vote)
                            end
                        end
                    end
                end
            end
        end
    end, "GET")
end

local function VotesUpdater()
    FetchVotes()
    SetTimeout(30000, VotesUpdater)
end

Citizen.CreateThread(function()
    VotesUpdater()
end)

RegisterServerEvent("sp-vote:newVoteOnline")
AddEventHandler("sp-vote:newVoteOnline", function(player)
    reward(player)
end)

RegisterServerEvent("sp-vote:newVoteOffline")
AddEventHandler("sp-vote:newVoteOffline", function(vote)
    debugPrint("sp-vote:newVoteOffline")
    MySQL.Async.execute("INSERT INTO sp_votes (voted_at, username) VALUES (@voted_at, @username)", {
        ["@voted_at"] = vote.voted_at,
        ["@username"] = vote.username
    }, function(rowsChanged)
        debugPrint(("New Vote Offline - Voted at: %s - Username: %s"):format(vote.voted_at, vote.username))
    end)
end)

RegisterServerEvent("sp-vote:checkVote")
AddEventHandler("sp-vote:checkVote", function()
    debugPrint("sp-vote:checkVote")
    local _source = source
    local playerName = GetPlayerName(_source)
    MySQL.Async.fetchAll("SELECT * FROM sp_votes WHERE username = @username", {
        ["@username"] = playerName
    }, function(result)
        if result[1] ~= nil then
            TriggerClientEvent("sp-vote:helpNotification", _source, ("Merci de votre Vote ! Récompense : ~g~$%s~s~"):format(Config.MoneyEarned))
            reward(_source)
            MySQL.Async.execute("DELETE FROM sp_votes WHERE username = @username", {
                ["@username"] = playerName
            }, function(rowsChanged)
                debugPrint(("Vote supprimé - Username: %s"):format(playerName))
            end)
        else
            debugPrint("not cfx name", playerName:lower())
            if verifRpNames(playerName:lower()) then
                debugPrint("player voted")
                TriggerClientEvent("sp-vote:helpNotification", _source, ("Merci de votre Vote ! Récompense : ~g~$%s~s~"):format(Config.MoneyEarned))
                reward(_source)
            else
                TriggerClientEvent("sp-vote:helpNotification", _source, "Vous n'avez pas encore voté ! Votez sur : serveur-prive.net")
            end
        end
    end)
end)
